#ifndef _FROG_ELEMENT_EVENT_CLUSTER_SISTRIP_H__
#define _FROG_ELEMENT_EVENT_CLUSTER_SISTRIP_H__

#include "FROG_Element_Event_Hit.h"
#include "FROG_Objects_Extended.h"


class FROG_Element_Event_Cluster_SiStrip : public FROG_Element_Base_With_DetId{
public :
    unsigned short firstStrip_;	float pitch_;
	std::vector<unsigned char> Ampl_;
	FROG_Objects_Extended*  frogObjects_;
	unsigned int nameId_;
	float charge_;

	virtual bool isCompactible(){ return false; }
	static unsigned int sizeOf(){ return FROG_Element_Base_With_DetId::sizeOf() + 1*sizeof(float) + 2*sizeof(unsigned short); }

    FROG_Element_Event_Cluster_SiStrip( unsigned int DetId,unsigned short firstStrip, float pitch):
    FROG_Element_Base_With_DetId(C_RECO_CLUSTER_SISTRIP, DetId),
    firstStrip_(firstStrip),pitch_(pitch){
            size_ = sizeOf();
    }    

    FROG_Element_Event_Cluster_SiStrip(FILE* pFile) : FROG_Element_Base_With_DetId(C_RECO_CLUSTER_SISTRIP)
    {
        size_ = sizeOf();
        fread(&detId_        ,sizeof(detId_)      ,1,pFile);
        fread(&firstStrip_   ,sizeof(firstStrip_) ,1,pFile);
		fread(&pitch_        ,sizeof(pitch_)      ,1,pFile);
    }

    unsigned int ReadComponents(FILE* pFile, unsigned int ToRead)
    {
		if(ToRead==0)return 0;						

		unsigned short NStrips;
		fread(&NStrips      ,sizeof(NStrips)    ,1,pFile);
		for(unsigned int i=0;i<NStrips;i++){
			unsigned char tmp;
			fread(&tmp      ,sizeof(tmp)    ,1,pFile);
			Ampl_.push_back(tmp);
		}

		return NStrips*sizeof(unsigned char);
    }


    virtual void write () {
		unsigned short NStrips = Ampl_.size();

        size_ = sizeOf() + NStrips*sizeof(unsigned char);
        data_ = new unsigned char[size_-6];
        data_ = FillBuffer( data_, &detId_,          sizeof(detId_));
        data_ = FillBuffer( data_, &firstStrip_,     sizeof(firstStrip_));
		data_ = FillBuffer( data_, &pitch_,          sizeof(pitch_));		
		data_ = FillBuffer( data_, &NStrips,         sizeof(NStrips));
		for(unsigned int i=0;i<NStrips;i++){				
			data_ = FillBuffer( data_, &Ampl_[i],        sizeof(Ampl_[i]));				
		}
        data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );
    }



	virtual void printInfos(char* buffer){		
		unsigned int charge=0;
		for(unsigned int i=0;i<Ampl_.size();i++){charge+=Ampl_[i];}
		sprintf(buffer,"SiStrip Cluster : DetId=%i firstStrip = %3i, clusterSize = %2i charge = %4iADCs",detId_, firstStrip_, (int)Ampl_.size(),charge);
	}

    #ifdef FROG_OPENGL	
    virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);
		if(charge_<style_->EMin_){return;}
		if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}

		
		if(Start_Display(UseDisplayList)){
			glLineWidth(style_->thickness_);
			glLoadName( nameId_ );			
			FROG_Light::SwitchLight_NoNormal();

			FROG_Element_Geom_TrackerMod* det = dynamic_cast<FROG_Element_Geom_TrackerMod*>(frogObjects_->frogGeom_->FindByDetId(detId_));
			if(!det){End_Display(UseDisplayList);return;}		

			float x,y,z;
			for(unsigned int i=0;i<Ampl_.size();i++){
				x = det->WidthX * (firstStrip_-1+i)*pitch_ / (2*sqrt(det->WidthX*det->WidthX + det->WidthY*det->WidthY + det->WidthZ*det->WidthZ));
				y = det->WidthY * (firstStrip_-1+i)*pitch_ / (2*sqrt(det->WidthX*det->WidthX + det->WidthY*det->WidthY + det->WidthZ*det->WidthZ));
				z = det->WidthZ * (firstStrip_-1+i)*pitch_ / (2*sqrt(det->WidthX*det->WidthX + det->WidthY*det->WidthY + det->WidthZ*det->WidthZ));

				if(style_->thickness_>0){
					glBegin (GL_LINES);
					glVertex3f(det->PosX+x*det->TrapezoidalParam+det->LengthX, det->PosY+y*det->TrapezoidalParam+det->LengthY, det->PosZ+z*det->TrapezoidalParam+det->LengthZ);
					glVertex3f(det->PosX+x-det->LengthX, det->PosY+y-det->LengthY, det->PosZ+z-det->LengthZ);
					glEnd(); 
				}
			}

			if(style_->marker_>0){
				glBindTexture(GL_TEXTURE_2D,frogObjects_->getMarker(style_->marker_));
				glEnable(GL_TEXTURE_2D);
				glTexEnvf( GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE );
				glEnable( GL_POINT_SPRITE_ARB );
			}

			if(style_->marker_>=0){
				glPointSize (style_->markerSize_);
				glBegin (GL_POINTS);
				glVertex3f(det->PosX+x*det->TrapezoidalParam, det->PosY+y*det->TrapezoidalParam, det->PosZ+z*det->TrapezoidalParam);
				glEnd(); 
			}

			if(style_->marker_>0){
				glDisable( GL_POINT_SPRITE_ARB );
				glDisable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D,0);
			}

			glLineWidth(1);
			FROG_Light::SwitchLight_Normal();
			if(style_->ShowDet_) det->displayLines();			

			End_Display(UseDisplayList);
		}
    }	
    #endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects_Extended*) frogObjects;		
			nameId_      = frogObjects_->frogEvent_->name_map.size();
			frogObjects_->frogEvent_->name_map.push_back(this);
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = mother_->style_;
			}else{												style_ = new FROG_Objects_Style();	}	
			
			for(unsigned int i=0;i<Ampl_.size();i++){charge_+=Ampl_[i];}			
			
			/*
			if(style_->EMin_<0){
				FROG_Element_Base_With_DetId* mother = dynamic_cast<FROG_Element_Base_With_DetId*>(mother_);
				if(mother)frogObjects_->frogCard_->GetFloat(&style_->EMin_,"Id_%i_AmplitudeMin", mother->DetId());
				if(mother)printf("Id_%i_AmplitudeMin", mother->DetId());
			}*/

			
		}		
	}
};

#endif
