#ifndef _FROG_ELEMENT_EVENT_SEGMENT_H__
#define _FROG_ELEMENT_EVENT_SEGMENT_H__

#include "FROG_Element_Base.h"
#include "FROG_Objects_Extended.h"

#ifdef FROG_OPENGL
	#include "../GL/glext.h"
#endif

class FROG_Element_Event_Segment : public FROG_Element_Base_With_DetId {
public :
    float x;  float y;  float z;  
	float dx; float dy; float dz;
	FROG_Objects_Extended*  frogObjects_;
	unsigned int nameId_;

    virtual bool isCompactible(){ return true; }

    static unsigned int sizeOf(){ return FROG_Element_Base_With_DetId::sizeOf() + 6*sizeof(float); }

    FROG_Element_Event_Segment( unsigned int DetId,
    float x_, float y_, float z_, float dx_, float dy_, float dz_):
    FROG_Element_Base_With_DetId(C_RECO_SEGMENT, DetId),
    x(x_),y(y_),z(z_),dx(dx_), dy(dy_), dz(dz_)
    {
            size_ = sizeOf();
    }

    FROG_Element_Event_Segment(FILE* pFile) : FROG_Element_Base_With_DetId(C_RECO_SEGMENT)
    {
            size_ = sizeOf();
            fread(&detId_        ,sizeof(detId_)      ,1,pFile);
            fread(&x             ,sizeof(x)           ,1,pFile);
            fread(&y             ,sizeof(y)           ,1,pFile);
            fread(&z             ,sizeof(z)           ,1,pFile);
            fread(&dx            ,sizeof(dx)          ,1,pFile);
            fread(&dy            ,sizeof(dy)          ,1,pFile);
            fread(&dz            ,sizeof(dz)          ,1,pFile);           
    }

    virtual void write () {
            size_ = sizeOf();
            data_ = new unsigned char[size_-6];
            data_ = FillBuffer( data_, &detId_,          sizeof(detId_));
            data_ = FillBuffer( data_, &x,               sizeof(x));
            data_ = FillBuffer( data_, &y,               sizeof(y));
            data_ = FillBuffer( data_, &z,               sizeof(z));
            data_ = FillBuffer( data_, &dx,              sizeof(dx));
            data_ = FillBuffer( data_, &dy,              sizeof(dy));
            data_ = FillBuffer( data_, &dz,              sizeof(dz));            
            data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );
    }

	virtual void printInfos(char* buffer){				
		sprintf(buffer,"Segment : DetId=%i DirX=%+7.2f DirY=%+7.2f DirZ=%+7.2f, X=%+7.2f Y=%+7.2f Z=%+7.2f",detId_, dx,dy,dz, x, y, z);		
	}

    #ifdef FROG_OPENGL	
    virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);
		if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}

		if(Start_Display(UseDisplayList)){
			glLoadName( nameId_ );			
			glLineWidth(style_->thickness_);

			FROG_Light::SwitchLight_NoNormal();
			glBegin (GL_LINES); 		
				glVertex3f (x,y,z); 
				glVertex3f (x+40*dx, y+40*dy, z+40*dz); 
			glEnd(); 
			FROG_Light::SwitchLight_Normal();

			
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
			if(FROG_Element_Base_With_DetId* det = dynamic_cast<FROG_Element_Base_With_DetId*>(frogObjects_->frogGeom_->FindByDetId(detId_))){                
					det->display(false);
			}		
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			glLineWidth(1);

			End_Display(UseDisplayList);
		}
    }
    #endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects_Extended*) frogObjects;
			nameId_      = frogObjects_->frogEvent_->name_map.size();
			frogObjects_->frogEvent_->name_map.push_back(this);
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = mother_->style_;
			}else{												style_ = new FROG_Objects_Style();	}
		}		
	}
};

#endif
