#ifndef _FROG_ELEMENT_EVENT_SIM_TRACK_H__
#define _FROG_ELEMENT_EVENT_SIM_TRACK_H__

#include "FROG_Element_Base.h"
#include "FROG_Element_Event_Sim_Vertex.h"
#include "FROG_Objects_Extended.h"

class FROG_Element_Event_Sim_Track : public FROG_Element_Base {
public :	
    int track_id;float Px;float Py; float Pz; float E;int Type;int parent_vertex;float charge;
	FROG_Objects_Extended*  frogObjects_;
	unsigned int nameId_;

	unsigned int MaxIteration_;
	float        MinSegLength_;
	float		 BField_[3];

	virtual bool isCompactible(){ return true; }

	static unsigned int sizeOf(){ return FROG_Element_Base::sizeOf() + 5*sizeof(float) +3*sizeof(int); }

	FROG_Element_Event_Sim_Track(
	int track_id_,float Px_,float Py_, float Pz_, float E_,int Type_,int parent_vertex_,float charge_):
	FROG_Element_Base(C_SIM_TRACK),
        track_id(track_id_), Px(Px_), Py(Py_), Pz(Pz_), E(E_), Type(Type_),parent_vertex(parent_vertex_),charge(charge_)
	{    
		size_ = sizeOf();		
	}

	FROG_Element_Event_Sim_Track(FILE* pFile) : FROG_Element_Base(C_SIM_TRACK)
	{   
		size_ = sizeOf();
		fread(&track_id         ,sizeof(track_id)       ,1,pFile);
		fread(&Px               ,sizeof(Px)             ,1,pFile);
		fread(&Py               ,sizeof(Py)             ,1,pFile);
		fread(&Pz               ,sizeof(Pz)             ,1,pFile);
		fread(&E                ,sizeof(E)              ,1,pFile);
		fread(&Type             ,sizeof(Type)           ,1,pFile);
		fread(&parent_vertex    ,sizeof(parent_vertex)  ,1,pFile);
		fread(&charge           ,sizeof(charge)  ,1,pFile);
	}

	virtual void write () {
		size_ = sizeOf();
		data_ = new unsigned char[size_-6];
		data_ = FillBuffer( data_, &track_id,		sizeof(track_id));
		data_ = FillBuffer( data_, &Px,				sizeof(Px));
		data_ = FillBuffer( data_, &Py,				sizeof(Py));
		data_ = FillBuffer( data_, &Pz,		        sizeof(Pz));
		data_ = FillBuffer( data_, &E,		        sizeof(E));
		data_ = FillBuffer( data_, &Type,    	    sizeof(Type));
		data_ = FillBuffer( data_, &parent_vertex,  sizeof(parent_vertex));
		data_ = FillBuffer( data_, &charge,    	    sizeof(charge));
		data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );		
	}

	virtual void printInfos(char* buffer){
		float PT = sqrt(Px*Px+Py*Py);
		sprintf(buffer,"SimTrack : TrackId= %6i Type = %+6i Charge = %+3.1f PT = %7.2f E=%7.2f",track_id, Type, charge, PT, E);
	}

	#ifdef FROG_OPENGL
	virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);
		if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}

		if(Start_Display(UseDisplayList)){

			double PT = sqrt(Px*Px+Py*Py);				 
			if(PT<style_->PtMin_){End_Display(UseDisplayList);return;}

			FROG_Element_Base* SimVtxColl = frogObjects_->frogEvent_->FindByDetId( EVTID_SIMVTX );
			if(!SimVtxColl){End_Display(UseDisplayList);return;}

			if(parent_vertex>=(int)SimVtxColl->getDaughterSize()){End_Display(UseDisplayList);return;}
			FROG_Element_Event_Sim_Vertex* vertex1 = dynamic_cast<FROG_Element_Event_Sim_Vertex*>(SimVtxColl->getDaughter(parent_vertex));
			if(!vertex1){End_Display(UseDisplayList);return;}

			
			FROG_Element_Event_Sim_Vertex* vertex2 = NULL;
			for(unsigned int i=0;i<SimVtxColl->getDaughterSize();i++){
				FROG_Element_Event_Sim_Vertex* tmp = dynamic_cast<FROG_Element_Event_Sim_Vertex*>(SimVtxColl->getDaughter(i));
				if(!tmp)continue;
					 
				if(tmp->parentTrack_id == track_id){				 
				 vertex2 = dynamic_cast<FROG_Element_Event_Sim_Vertex*>(SimVtxColl->getDaughter(i));
				 break;
				}			 
			}

			int TRACKING_RADIUS = 129;
			int TRACKING_LENGTH = 300;

			if(sqrt(vertex1->x*vertex1->x+vertex1->y*vertex1->y)>TRACKING_RADIUS){End_Display(UseDisplayList);return;}
			if(fabs(vertex1->z)>TRACKING_LENGTH){End_Display(UseDisplayList);return;}

	//				   printf("Track %+05i\tType=%+05i Pt=%6.2f\n",track_id,Type,PT);
	//                 printf("Track %+05i\tVertex1 = %6.2f %6.2f %6.2f\n",track_id,vertex1->x,vertex1->y,vertex1->z);
	//		if(vertex2)printf("Track %+05i\tVertex2 = %6.2f %6.2f %6.2f\n",track_id,vertex2->x,vertex2->y,vertex2->z);	


			
			glLineWidth(style_->thickness_);
			glLoadName( nameId_ );
			

			double q  = charge;
			double px = Px / 0.003;
			double py = Py / 0.003;
			double pz = Pz / 0.003;
			double pt = sqrt(px*px+py*py);
			double p  = sqrt(px*px+py*py+pz*pz);

			if(q!=0){
			double e  = E  / 0.003;
			double M  = sqrt(e*e - (px*px + py*py + pz*pz) );               

			double vx = px/M;
			double vy = py/M;
			double vz = pz/M;

			double Bx = BField_[0];
			double By = BField_[1];
			double Bz = BField_[2];

			double ax =  (q/M)*(Bz*vy - By*vz);
			double ay =  (q/M)*(Bx*vz - Bz*vx);
			double az =  (q/M)*(By*vx - Bx*vy);

			double dt = 1/p;
			if(pt<266 && vz < 0.0012)	dt = fabs(0.001/vz);

			double xold=vertex1->x;		double x=xold;
			double yold=vertex1->y;		double y=yold;
			double zold=vertex1->z;		double z=zold;

			double VTold = sqrt(vx*vx+vy*vy);
			double DVertex;
			if(vertex2!=NULL)DVertex = ((x-vertex2->x)*(x-vertex2->x)) + ((y-vertex2->y)*(y-vertex2->y)) + ((z-vertex2->z)*(z-vertex2->z)) ;
			
			glDisable(GL_LINE_SMOOTH);

			unsigned int k = 0;
			bool ShouldPropagate = true;

			FROG_Light::SwitchLight_NoNormal();
			while(ShouldPropagate && k<MaxIteration_){
				k++;

				vx += ax*dt;
				vy += ay*dt;
				vz += az*dt;

				double VTratio = VTold/sqrt(vx*vx+vy*vy);
				vx *= VTratio;
				vy *= VTratio;                  

				ax = (q/M)*(Bz*vy - By*vz);
				ay = (q/M)*(Bx*vz - Bz*vx);
				az = (q/M)*(By*vx - Bx*vy);

				x  += vx*dt;
				y  += vy*dt;
				z  += vz*dt;

				if(vertex2!=NULL){
					double newDVertex = ((x-vertex2->x)*(x-vertex2->x)) + ((y-vertex2->y)*(y-vertex2->y)) + ((z-vertex2->z)*(z-vertex2->z));
					if(newDVertex<DVertex){		
						DVertex = newDVertex;
					}else{
						// Just make sure that the SimTrack really go from Vertex1 To Vertex2 if Vertex2 Exist!
						x = vertex2->x;
						y = vertex2->y;
						z = vertex2->z;
						ShouldPropagate = false;
					}
				}

				if( (x*x+y*y) > TRACKING_RADIUS*TRACKING_RADIUS ){ x /= (x*x+y*y)/(TRACKING_RADIUS*TRACKING_RADIUS); y /= (x*x+y*y)/(TRACKING_RADIUS*TRACKING_RADIUS); break;}
				if( fabs(z)>TRACKING_LENGTH)break;
				if( fabs((x*x+y*y+z*z) - (xold*xold+yold*yold+zold*zold))<MinSegLength_ && ShouldPropagate)continue;

				glBegin (GL_LINES);             
					glVertex3f (xold,yold,zold); 
					glVertex3f (x,y,z); 
				glEnd();     

				xold = x;
				yold = y;
				zold = z;
			}

			// Get The AntiAliasing back
			glEnable(GL_LINE_SMOOTH);
			}else{          
			if(vertex2 && sqrt(vertex2->x*vertex2->x+vertex2->y*vertex2->y)<TRACKING_RADIUS && fabs(vertex2->z)<TRACKING_LENGTH ){						
				glBegin (GL_LINES);             
					glVertex3f (vertex1->x,vertex1->y,vertex1->z); 
					glVertex3f (vertex2->x,vertex2->y,vertex2->z);
				glEnd();          					
			}else{                        
				double t1 = TRACKING_RADIUS/sqrt(px*px+py*py);
				double t2 = TRACKING_LENGTH/fabs(pz);
				if(t2<t1)t1=t2;

				glBegin (GL_LINES);             
						glVertex3f (vertex1->x,vertex1->y,vertex1->z); 
						glVertex3f (px*t1,py*t1,pz*t1); 
				glEnd();
			}
			}               
			glLineWidth(1);
			FROG_Light::SwitchLight_Normal();

			End_Display(UseDisplayList);
		}
	}
	#endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects_Extended*) frogObjects;
			nameId_      = frogObjects_->frogEvent_->name_map.size();
			frogObjects_->frogEvent_->name_map.push_back(this);			
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL){	style_ = new FROG_Objects_Style(mother_->style_);
			}else{					style_ = new FROG_Objects_Style();				}	
	
			if(charge>0 ) frogObjects_->frogCard_->GetColor(style_->color_,"SimTrack_Charge_+_Color");
			if(charge<0 ) frogObjects_->frogCard_->GetColor(style_->color_,"SimTrack_Charge_-_Color");
			if(charge==0) frogObjects_->frogCard_->GetColor(style_->color_,"SimTrack_Charge_0_Color");		              
						  frogObjects_->frogCard_->GetColor(style_->color_,"SimTrack_PdgId_%i_Color", Type);

	//		if(charge>0 ) frogObjects_->frogCard_->GetFloat(&style_->ptMin_,"SimTrack_Charge_+_PtMin");
	//		if(charge<0 ) frogObjects_->frogCard_->GetFloat(&style_->ptMin_,"SimTrack_Charge_-_PtMin");
	//		if(charge==0) frogObjects_->frogCard_->GetFloat(&style_->ptMin_,"SimTrack_Charge_0_PtMin");			             
	//				      frogObjects_->frogCard_->GetFloat(&style_->ptMin_,"SimTrack_PdgId_%i_PtMin", Type);

			if(charge>0 )frogObjects_->frogCard_->GetFloat(&style_->thickness_,"SimTrack_Charge_+_Thickness");
			if(charge<0 )frogObjects_->frogCard_->GetFloat(&style_->thickness_,"SimTrack_Charge_-_Thickness");
			if(charge==0)frogObjects_->frogCard_->GetFloat(&style_->thickness_,"SimTrack_Charge_0_Thickness");			         
			             frogObjects_->frogCard_->GetFloat(&style_->thickness_,"SimTrack_PdgId_%i_Thickness", Type);

			if(charge!=0){
				MaxIteration_ = 1000000;
				frogObjects_->frogCard_->GetUInt(&MaxIteration_,"SimTrack_MaxIt");

				MinSegLength_ = 50;
				frogObjects_->frogCard_->GetFloat(&MinSegLength_,"SimTrack_MinSegLength");

				BField_[0] = 0.0;	BField_[1] = 0.0;	BField_[2] = 4.0;
				frogObjects_->frogCard_->GetColor(BField_,"SimTrack_BField");
			}
		}
	}
};

#endif
