#ifndef _FROG_ELEMENT_EVENT_SIM_VERTEX_H__
#define _FROG_ELEMENT_EVENT_SIM_VERTEX_H__

#include "FROG_Element_Base.h"
#include "FROG_Objects_Extended.h"

#ifdef FROG_OPENGL
	#include "../GL/glext.h"
#endif

class FROG_Element_Event_Sim_Vertex : public FROG_Element_Base {
public :	
    float x; float y; float z; int parentTrack_id;
	FROG_Objects_Extended*  frogObjects_;
	
	virtual bool isCompactible(){ return true; }

	static unsigned int sizeOf(){ return FROG_Element_Base::sizeOf() + 3*sizeof(float) +sizeof(int); }

	FROG_Element_Event_Sim_Vertex(
	float x_,float y_, float z_, int parentTrack_id_):
	FROG_Element_Base(C_SIM_VERTEX),
        x(x_), y(y_), z(z_),parentTrack_id(parentTrack_id_)
	{    
		size_ = sizeOf();
	}

	FROG_Element_Event_Sim_Vertex(FILE* pFile) : FROG_Element_Base(C_SIM_VERTEX)
	{   
		size_ = sizeOf();
		fread(&x               ,sizeof(x)				,1,pFile);
		fread(&y               ,sizeof(y)				,1,pFile);
		fread(&z               ,sizeof(z)				,1,pFile);
		fread(&parentTrack_id  ,sizeof(parentTrack_id)  ,1,pFile);
	}

	virtual void write () {
		size_ = sizeOf();
		data_ = new unsigned char[size_-6];
		data_ = FillBuffer( data_, &x,				sizeof(x));
		data_ = FillBuffer( data_, &y,				sizeof(y));
		data_ = FillBuffer( data_, &z,		        sizeof(z));
		data_ = FillBuffer( data_, &parentTrack_id, sizeof(parentTrack_id));
		data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );		
	}

	virtual void printInfos(char* buffer){		
		sprintf(buffer,"SimVertex : x=%+7.2f y=%+7.2f z=%+8.2f ParentTrackId=%7i",x,y,z, parentTrack_id);
	}

	#ifdef FROG_OPENGL
	virtual void display(bool UseDisplayList=true, float* color=NULL){		
		// first Init Colors & Style!
		init(NULL);
		if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}
		FROG_Light::SwitchLight_NoNormal();


		if(Start_Display(UseDisplayList)){
			if(style_->marker_>0){
				glBindTexture(GL_TEXTURE_2D,frogObjects_->getMarker(style_->marker_));
				glEnable(GL_TEXTURE_2D);
				glTexEnvf( GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE );
				glEnable( GL_POINT_SPRITE_ARB );	
			}

			glPointSize (style_->markerSize_); 		
			glBegin (GL_POINTS);
			glVertex3f(x, y, z);
			glEnd(); 

			if(style_->marker_>0){
				glDisable( GL_POINT_SPRITE_ARB );
				glDisable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D,0);			
			}		

			FROG_Light::SwitchLight_Normal();
			End_Display(UseDisplayList);
		}
	}
	#endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects_Extended*) frogObjects;			
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = mother_->style_;
			}else{												style_ = new FROG_Objects_Style();	}	
		}				
	}
};

#endif

