#ifndef _FROG_ELEMENT_EVENT_TRACK_H__
#define _FROG_ELEMENT_EVENT_TRACK_H__

#include "FROG_Element_Base.h"
#include "FROG_Element_Event_Hit.h"
#include "FROG_Objects_Extended.h"

class FROG_Element_Event_Track : public FROG_Element_Base {
protected:
	char coll_; float p_; float pt_; float chi2_;
	FROG_Objects_Extended*  frogObjects_;
	unsigned int nameId_;

public:
	virtual bool isCompactible(){ return false; }
	static unsigned int sizeOf(){ return FROG_Element_Base::sizeOf() + 3*sizeof(float) + 1*sizeof(char); }

	FROG_Element_Event_Track(char Coll, float P, float Pt, float Chi2) : FROG_Element_Base(C_RECO_TRACK),
	coll_(Coll), p_(P),pt_(Pt),chi2_(Chi2){
		size_ = sizeOf();
	}

	FROG_Element_Event_Track(FILE* pFile) : FROG_Element_Base(C_RECO_TRACK){		
        size_ = sizeOf();		
		fread(&coll_ ,sizeof(coll_),1,pFile);
		fread(&p_    ,sizeof(p_)   ,1,pFile);
		fread(&pt_   ,sizeof(pt_)  ,1,pFile);
		fread(&chi2_ ,sizeof(chi2_),1,pFile);
    }

    virtual void write(){
		//printf("size_ = %i\n",size_);
		//printf("NHits = %i X %i = %i\n",hits_.size(),hits_[0]->sizeOf(),hits_.size()*hits_[0]->sizeOf());		
		std::vector< std::pair<unsigned int, unsigned int > >* blockOfDaughters = write_init();
		if(blockOfDaughters==NULL)return;				
		data_ = FillBuffer(data_, &coll_, sizeof(coll_));
		data_ = FillBuffer(data_, &p_   , sizeof(p_));
		data_ = FillBuffer(data_, &pt_  , sizeof(pt_));
		data_ = FillBuffer(data_, &chi2_, sizeof(chi2_));
		write_daughters(blockOfDaughters);
	}

	virtual void printInfos(char* buffer){		
		sprintf(buffer,"Track : P=%7.2f Pt=%7.2f Chi2=%5.2f",p_,pt_,chi2_);
	}

    #ifdef FROG_OPENGL
    virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);    
		if(pt_<style_->PtMin_){End_Display(UseDisplayList);return;}
		if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}
		
		if(Start_Display(UseDisplayList)){
			glLoadName( nameId_ );		
			glLineWidth(style_->thickness_);			
			

//			float global_ambient[] = { 1.0f, 1.0f, 1.0f, 1.0f };
//			glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);

			// draw the track
			

			FROG_Light::SwitchLight_NoNormal();
			glBegin (GL_LINE_STRIP);        
			for( unsigned int j=0; j < daughters_.size(); j++ ){				
				FROG_Element_Event_Hit* hit = dynamic_cast<FROG_Element_Event_Hit*>(daughters_[j]);
				if(hit==NULL)continue;
				glVertex3f (hit->x,hit->y,hit->z);				
			}
			glEnd();
			glLineWidth(1);			

			// display all the hits and the module associated
			for( unsigned int j=0; j < daughters_.size(); j++ ){
				FROG_Element_Event_Hit* hit = dynamic_cast<FROG_Element_Event_Hit*>(daughters_[j]);
				if(hit==NULL)continue;

				if(style_->marker_>0){
					if(style_->marker_==100){glBindTexture(GL_TEXTURE_2D,frogObjects_->getMarker(style_->marker_+j));
					}else{glBindTexture(GL_TEXTURE_2D,frogObjects_->getMarker(style_->marker_));}
					glEnable(GL_TEXTURE_2D);
					glTexEnvf( GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE );
					glEnable( GL_POINT_SPRITE_ARB );
				}

				glPointSize (style_->markerSize_); 		

				glBegin (GL_POINTS);
				glColor4fv (style_->color_);			
				glVertex3f(hit->x, hit->y, hit->z);
				glEnd(); 


				if(style_->marker_>0){
					glDisable( GL_POINT_SPRITE_ARB );
					glDisable(GL_TEXTURE_2D);
					glBindTexture(GL_TEXTURE_2D,0);
				}
			}
			FROG_Light::SwitchLight_Normal();


			if(style_->ShowDet_){
				for( unsigned int j=0; j < daughters_.size(); j++ ){	
					FROG_Element_Event_Hit* hit = dynamic_cast<FROG_Element_Event_Hit*>(daughters_[j]);
					if(hit==NULL)continue;
					if(FROG_Element_Base_With_DetId* det = dynamic_cast<FROG_Element_Base_With_DetId*>(frogObjects_->frogGeom_->FindByDetId(hit->DetId()))){                    
						det->display(false);
					}						
				}
			}

			End_Display(UseDisplayList);
		}
    }
    #endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects_Extended*) frogObjects;
			nameId_      = frogObjects_->frogEvent_->name_map.size();
			frogObjects_->frogEvent_->name_map.push_back(this);
			for(unsigned int i=0;i<daughters_.size();i++)	daughters_[i]->init(frogObjects);
		}else if(frogObjects==NULL && frogObjects_ != NULL && style_ == NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = new FROG_Objects_Style(mother_->style_);
			}else{												style_ = new FROG_Objects_Style();	}	

			frogObjects_->frogCard_->GetColor(style_->color_,"Id_%i_Color%i",((FROG_Element_Base_With_DetId*)mother_)->DetId(),style_->Index_%5);			
		}	

	}

};
#endif
