#ifndef _FROG_ELEMENT_PRIMITIVE_PARTIALSPHERE_H__
#define _FROG_ELEMENT_PRIMITIVE_PARTIALSPHERE_H__

#include "FROG_Element_Base.h"

class FROG_Element_Primitive_PartialSphere : public FROG_Element_Base_With_DetId {
public :
	float Radius;
	float PosX;			 float PosY;    float PosZ;	
	float Phi1;			 float Phi2;	
	float Theta1;		 float Theta2;	
	unsigned short NPhi; unsigned short NTheta;	float ROT;
	FROG_Objects*  frogObjects_;

	virtual bool isCompactible(){ return true; }

	static unsigned int sizeOf(){ return FROG_Element_Base_With_DetId::sizeOf() + 9*sizeof(float) + 2*sizeof(unsigned short);}

	FROG_Element_Primitive_PartialSphere(
	unsigned int detId,		float radius,
	float posX,				float posY,    float posZ,
	float phi1,				float phi2,
	float theta1,			float theta2,
	unsigned short nPhi=10, unsigned short nTheta=5, float rot=0) :
	FROG_Element_Base_With_DetId(C_PRIMITIVE_PARTIALSPHERE,detId),
	Radius(radius),
	PosX(posX),			PosY(posY),			PosZ(posZ),
	Phi1(phi1),			Phi2(phi2),
	Theta1(theta1),		Theta2(theta2),
	NPhi(nPhi),			NTheta(nTheta), ROT(rot)
	{    
		size_ = sizeOf();
	}

	FROG_Element_Primitive_PartialSphere(FILE* pFile) : FROG_Element_Base_With_DetId(C_PRIMITIVE_PARTIALSPHERE)
	{   
		size_ = sizeOf();
		fread(&detId_           ,sizeof(detId_)			 ,1,pFile);				
		fread(&Radius           ,sizeof(Radius)          ,1,pFile);
		fread(&PosX             ,sizeof(PosX)            ,1,pFile);
		fread(&PosY             ,sizeof(PosY)            ,1,pFile);
		fread(&PosZ             ,sizeof(PosZ)            ,1,pFile);
		fread(&Phi1             ,sizeof(Phi1)            ,1,pFile);
		fread(&Phi2             ,sizeof(Phi2)            ,1,pFile);
		fread(&Theta1           ,sizeof(Theta1)          ,1,pFile);
		fread(&Theta2           ,sizeof(Theta2)          ,1,pFile);
		fread(&NPhi             ,sizeof(NPhi)            ,1,pFile);
		fread(&NTheta           ,sizeof(NTheta)          ,1,pFile);
		fread(&ROT              ,sizeof(ROT)             ,1,pFile);
	}

	virtual void write () {
		size_ = sizeOf();
		data_ = new unsigned char[size_-6];
		data_ = FillBuffer(	data_, &detId_,				sizeof(detId_));
		data_ = FillBuffer(	data_, &Radius,				sizeof(Radius));
		data_ = FillBuffer( data_, &PosX,				sizeof(PosX));
		data_ = FillBuffer( data_, &PosY,				sizeof(PosY));
		data_ = FillBuffer( data_, &PosZ,				sizeof(PosZ));
		data_ = FillBuffer( data_, &Phi1,				sizeof(Phi1));
		data_ = FillBuffer( data_, &Phi2,				sizeof(Phi2));
		data_ = FillBuffer( data_, &Theta1,				sizeof(Theta1));
		data_ = FillBuffer( data_, &Theta2,				sizeof(Theta2));
		data_ = FillBuffer( data_, &NPhi,				sizeof(NPhi));
		data_ = FillBuffer( data_, &NTheta,				sizeof(NTheta));
		data_ = FillBuffer( data_, &ROT,				sizeof(ROT));
		data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );		
	}

	#ifdef FROG_OPENGL
	virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);

		glLineWidth(style_->thickness_);
        if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}
		glLoadName( detId_ );
		
		double dphi =	6.283185307179586476925286766559 /NPhi;
		double dthe =	3.1415926535897932384626433832795/NTheta;

		glPushMatrix();
		glTranslatef(PosX,PosY,PosZ);
		glRotatef(ROT,0,1,0);

		glBegin (GL_QUADS); 
			for(double the=Theta1; the<=Theta2-dthe;the+=dthe){
			for(double phi=Phi1;   phi<=Phi2  -dphi;phi+=dphi){			
				glVertex3f (Radius*cos(the)*cos(phi)			,Radius*cos(the)*sin(phi)			 ,Radius*sin(the));		// OUTER FACE
				glVertex3f (Radius*cos(the)*cos(phi+dphi)		,Radius*cos(the)*sin(phi+dphi)		 ,Radius*sin(the)); 
				glVertex3f (Radius*cos(the+dthe)*cos(phi+dphi)  ,Radius*cos(the+dthe)*sin(phi+dphi)  ,Radius*sin(the+dthe)); 
				glVertex3f (Radius*cos(the+dthe)*cos(phi)		,Radius*cos(the+dthe)*sin(phi)		 ,Radius*sin(the+dthe));

				glVertex3f (Radius*cos(the)*cos(phi)			,Radius*cos(the)*sin(phi)			 ,Radius*sin(the));		// INNER FACE
				glVertex3f (Radius*cos(the+dthe)*cos(phi)		,Radius*cos(the+dthe)*sin(phi)		 ,Radius*sin(the+dthe)); 
				glVertex3f (Radius*cos(the+dthe)*cos(phi+dphi)  ,Radius*cos(the+dthe)*sin(phi+dphi)  ,Radius*sin(the+dthe)); 
				glVertex3f (Radius*cos(the)*cos(phi+dphi)		,Radius*cos(the)*sin(phi+dphi)		 ,Radius*sin(the)); 							}}		
		glEnd(); 

		glPopMatrix();
	}
	#endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects*) frogObjects;			
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = new FROG_Objects_Style(mother_->style_);
			}else{												style_ = new FROG_Objects_Style();	}	

			frogObjects_->frogCard_->GetColor( style_->color_     ,"Id_%i_Color"		,detId_);
			frogObjects_->frogCard_->GetFloat(&style_->thickness_ ,"Id_%i_Thickness"	,detId_);
		}
	}
};
#endif
