#ifndef _FROG_ELEMENT_PRIMITIVE_SPHERE_H__
#define _FROG_ELEMENT_PRIMITIVE_SPHERE_H__

#include "FROG_Element_Base.h"

class FROG_Element_Primitive_Sphere : public FROG_Element_Base_With_DetId {
public :
	float Radius;
	float PosX;			 float PosY;    float PosZ;	
	unsigned short NPhi; unsigned short NTheta;	
	FROG_Objects*  frogObjects_;
	
	virtual bool isCompactible(){ return true; }

	static unsigned int sizeOf(){ return FROG_Element_Base_With_DetId::sizeOf() + 4*sizeof(float) + 2*sizeof(unsigned short);}

	FROG_Element_Primitive_Sphere(
	unsigned int detId,		float radius,
	float posX,				float posY,    float posZ,
	unsigned short nPhi=10, unsigned short nTheta=5) :
	FROG_Element_Base_With_DetId(C_PRIMITIVE_SPHERE,detId),
	Radius(radius),
	PosX(posX),			PosY(posY),			PosZ(posZ),
	NPhi(nPhi),			NTheta(nTheta)
	{    
		size_ = sizeOf();
	}

	FROG_Element_Primitive_Sphere(FILE* pFile) : FROG_Element_Base_With_DetId(C_PRIMITIVE_SPHERE)
	{   
		size_ = sizeOf();
		fread(&detId_           ,sizeof(detId_)			 ,1,pFile);				
		fread(&Radius           ,sizeof(Radius)          ,1,pFile);
		fread(&PosX             ,sizeof(PosX)            ,1,pFile);
		fread(&PosY             ,sizeof(PosY)            ,1,pFile);
		fread(&PosZ             ,sizeof(PosZ)            ,1,pFile);
		fread(&NPhi             ,sizeof(NPhi)            ,1,pFile);
		fread(&NTheta           ,sizeof(NTheta)          ,1,pFile);
	}

	virtual void write () {
		size_ = sizeOf();
		data_ = new unsigned char[size_-6];
		data_ = FillBuffer(	data_, &detId_,				sizeof(detId_));
		data_ = FillBuffer(	data_, &Radius,				sizeof(Radius));
		data_ = FillBuffer( data_, &PosX,				sizeof(PosX));
		data_ = FillBuffer( data_, &PosY,				sizeof(PosY));
		data_ = FillBuffer( data_, &PosZ,				sizeof(PosZ));
		data_ = FillBuffer( data_, &NPhi,				sizeof(NPhi));
		data_ = FillBuffer( data_, &NTheta,				sizeof(NTheta));
		data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );		
	}

	#ifdef FROG_OPENGL
	virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);

		glLineWidth(style_->thickness_);
        if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}
		glLoadName( detId_ );
		
		double dphi =	6.283185307179586476925286766559 /NPhi;
		double dthe =	3.1415926535897932384626433832795/NTheta;
		glBegin (GL_QUADS); 
			for(double the=0; the<=2*3.1416-dthe;the+=dthe){
			for(double phi=0; phi<=6.2831;phi+=dphi){								
				glVertex3f (PosX+Radius*cos(the)*cos(phi)			,PosY+Radius*cos(the)*sin(phi)			 ,PosZ+Radius*sin(the)); 
				glVertex3f (PosX+Radius*cos(the+dthe)*cos(phi)		,PosY+Radius*cos(the+dthe)*sin(phi)		 ,PosZ+Radius*sin(the+dthe));
				glVertex3f (PosX+Radius*cos(the+dthe)*cos(phi+dphi) ,PosY+Radius*cos(the+dthe)*sin(phi+dphi) ,PosZ+Radius*sin(the+dthe)); 
				glVertex3f (PosX+Radius*cos(the)*cos(phi+dphi)		,PosY+Radius*cos(the)*sin(phi+dphi)		 ,PosZ+Radius*sin(the)); 											}}		
		glEnd(); 
	}
	#endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects*) frogObjects;			
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = new FROG_Objects_Style(mother_->style_);
			}else{												style_ = new FROG_Objects_Style();	}	

			frogObjects_->frogCard_->GetColor( style_->color_     ,"Id_%i_Color"		,detId_);
			frogObjects_->frogCard_->GetFloat(&style_->thickness_ ,"Id_%i_Thickness"	,detId_);
		}
	}
};
#endif
