// FROG_Events.cpp: implementation of the FROG_Events class.
//
//////////////////////////////////////////////////////////////////////

#include "FROG_Events.h"

void FROG_Events::AddEvent(FROG_Element_Event* event){
	prim->addDaughter(event);
}

void FROG_Events::Load (const char* path)
{
	if(frogObjects_ && frogObjects_->frogEvent_){	
		frogObjects_->frogEvent_->uninit();
		frogObjects_->frogEvent_->deleteDaughters();
		frogObjects_->frogEvent_ = NULL;
	}
	if(prim){ delete prim; prim=NULL;}

	FilePath_ = path;

	FILE* pFile = fopen(FilePath_,"rb" );
	if(pFile==NULL){
		printf("The file %s can not be opened !\n",FilePath_);
		if(prim){ delete prim; prim=NULL;}
		return;
	}

	prim = new FROG_Element_Base(C_PRIMARY);
	FROG_ELEMENT::Read(pFile,prim,0);
	//FROG_ELEMENT::PrintTree(prim);
	fclose(pFile);

	const std::vector<FROG_Element_Base*>& dau = prim->daughters();
	for( unsigned int i=0; i<dau.size(); ++i) {			
		if( dau[i]->type() == C_EVENT || dau[i]->type() == C_EVENTR || dau[i]->type() == C_EVENTRT ){
			FROG_Element_Event* evt = dynamic_cast<FROG_Element_Event*>(dau[i]);
			if(evt==NULL)continue;
			//evt->FillMap();
		}
	}
}

void FROG_Events::Save (const char* path, bool CompressFiles)
{
	prim->write();
	FILE* pFile = fopen(path,"wb" );
		unsigned short prim_type = prim->type();
		unsigned int   prim_size = prim->size();
		fwrite(&prim_type,sizeof(prim_type),1,pFile);
		fwrite(&prim_size,sizeof(prim_size),1,pFile);
		fwrite(prim->data(),prim->size()-6,1,pFile);
	fclose(pFile);
	prim->clearData();


	#ifdef FROG_ZLIB
		if(CompressFiles){
			char* path2 = new char[strlen(path)];
			strcpy(path2,path);
			FROG_ZLIB_Deflate(path2);
			delete path2;
		}
	#endif
}


bool FROG_Events::SaveInLive (const char* path, bool CloseFile, bool CompressFiles, unsigned int MaxSize)
{
	bool MaxFileHasBeenReached = false;

	FILE* pFile;
	if(CurrentSize_==0){				
		//pFile = fopen("InLive.vis","wb");
		pFile = fopen(path,"wb");
		unsigned short prim_type = prim->type();
		unsigned int   prim_size = 6;
		fwrite(&prim_type,sizeof(prim_type),1,pFile);
		fwrite(&prim_size,sizeof(prim_size),1,pFile);
		CurrentSize_ += prim_size;	

		for(unsigned int i=0;i<prim->getDaughterSize();i++){
			FROG_Element_Base* dau = prim->getDaughter(i);
			if(dau==NULL)continue;			

			if(dau->data()==NULL)dau->write();
			unsigned short dau_type = dau->type();
			unsigned int   dau_size = dau->size();
			fwrite(&dau_type  ,sizeof(dau_type),1,pFile);
			fwrite(&dau_size  ,sizeof(dau_size),1,pFile);
			fwrite(dau->data(),dau->size()-6   ,1,pFile);
			CurrentSize_ += dau->size();
		}
		prim->deleteDaughters();		
	}else{
		//pFile = fopen("InLive.vis","ab+" );
		pFile = fopen(path,"ab+" );
		fseek(pFile,CurrentSize_,SEEK_SET);

		bool DeletedDaughter;
		do{
			DeletedDaughter = false;
			for(unsigned int i=0;i<prim->getDaughterSize()&&!MaxFileHasBeenReached&&!DeletedDaughter;i++){
				FROG_Element_Base* dau = prim->getDaughter(i);
				if(dau==NULL)continue;
				if(dau->data()==NULL)dau->write();
				if(CurrentSize_ + dau->size()>=MaxSize){
					MaxFileHasBeenReached=true;
//					for(unsigned j=0;j<i;j++){prim->deleteDaughter(i);}
				}
				if(MaxFileHasBeenReached)continue;

				unsigned short dau_type = dau->type();
				unsigned int   dau_size = dau->size();
				fwrite(&dau_type  ,sizeof(dau_type),1,pFile);
				fwrite(&dau_size  ,sizeof(dau_size),1,pFile);
				fwrite(dau->data(),dau->size()-6   ,1,pFile);
				CurrentSize_ += dau->size();	
				prim->deleteDaughter(i);
				DeletedDaughter = true;
			}
		}while(!MaxFileHasBeenReached && DeletedDaughter);
	}		
	fclose(pFile);	

	//pFile = fopen("InLive.vis","rb+" );
	pFile = fopen(path,"rb+" );
		fseek(pFile,2,SEEK_SET);
		fwrite(&CurrentSize_,sizeof(CurrentSize_),1,pFile);
	fclose(pFile);

	if(MaxFileHasBeenReached || CloseFile){				
		//remove (path);
		//rename ("InLive.vis", path);
		//copy("InLive.vis", path);

		#ifdef FROG_ZLIB
			if(CompressFiles){
				char* path2 = new char[strlen(path)];
				strcpy(path2,path);
				FROG_ZLIB_Deflate(path2);
				delete path2;
			}
		#endif

		CurrentSize_ = 0;
		return false;
	}

	return true;
}


FROG_Element_Event* FROG_Events::GoToEvent(int i)
{
	if(frogObjects_->frogEvent_){			
		frogObjects_->frogEvent_->uninit();
		frogObjects_->frogEvent_->deleteDaughters();
	}

	if(i<0 || (prim && i>=(int)prim->getDaughterSize())){		
		Load(FilePath_);
		if(prim)Current_Event = prim->getDaughterSize()-1; 
	}else{
		Current_Event=i;	
	}

	if(!prim)return NULL;
	frogObjects_->setEvent(dynamic_cast<FROG_Element_Event*>(prim->getDaughter(Current_Event)));

	FILE* pFile = fopen(FilePath_,"rb" );
		if(pFile==NULL){
			printf("The file %s can not be opened !\n",FilePath_);
			return NULL;
		}
		fseek (pFile,((FROG_Element_Event*)frogObjects_->frogEvent_)->EventPosInFile(),SEEK_SET);
		FROG_ELEMENT::Read(pFile,frogObjects_->frogEvent_,((FROG_Element_Event*)frogObjects_->frogEvent_)->EventNBytesToRead());			
	fclose(pFile);

	frogObjects_->frogEvent_->FillMap();
	prim->init((void*)frogObjects_);		
	updateEventToDisplay();
	updateOpenedMenu();
	
	return frogObjects_->frogEvent_;
}
FROG_Element_Event* FROG_Events::NextEvent()
{
	if(!prim)return NULL;	
	if(Current_Event+1<prim->getDaughterSize()){	Current_Event++;
	}else{
		Load(FilePath_);
		if(Current_Event+1<prim->getDaughterSize()){Current_Event++;}else{Current_Event=0;}
	}
	return GoToEvent(Current_Event);	 
}
FROG_Element_Event* FROG_Events::PreviousEvent()
{
	if(!prim)return NULL;
	if(Current_Event!=0){	Current_Event--;	
	}else{
		Load(FilePath_);
		Current_Event = prim->getDaughterSize()-1; 
	}
	return GoToEvent(Current_Event);	 
}

void FROG_Events::Init(FROG_Objects_Extended* frogObjects)
{
	if(!prim)return;
	frogObjects_ = frogObjects;
}

void FROG_Events::readEventToDisplay(FROG_ReadCards* Card){
	if(Card->IsTagPresent("EventToDisplay"))
		EventToDisplay  = Card->GetUIntVector("EventToDisplay");		
}


void FROG_Events::keepEventToDisplay(FROG_Element_Base* mother=NULL){
	if(mother==NULL){
		//if(frogObjects_->frogEvent_==NULL)return;
		mother = frogObjects_->frogEvent_;
		EventToDisplay.clear();
	}

	FROG_Element_Base_With_DetId* FEBD = dynamic_cast<FROG_Element_Base_With_DetId*>(mother);
	if(mother->getDisplay()==1 && FEBD){		
		EventToDisplay.push_back(FEBD->DetId());
	}else if(mother->getDisplay()!=0){
		for(unsigned int i=0;i<mother->getDaughterSize();i++){
			keepEventToDisplay(mother->getDaughter(i));
		}
	}
	
}



void FROG_Events::updateEventToDisplay(unsigned int EvtIdToFlip)
{	
	if(!prim)return;

	if(EvtIdToFlip==0){
		if(frogObjects_->frogEvent_->getDisplay()!=0)prim->setDisplay(0);
		for(unsigned int i=0;i<EventToDisplay.size();i++){			
			FROG_Element_Base* tmp = frogObjects_->frogEvent_->FindByDetId(EventToDisplay[i]);
			if(tmp){			
				tmp->setDisplay(1);
			}
		}
		frogObjects_->frogEvent_->setDisplay(2);
		return;
	}else{
		FROG_Element_Base* tmp = frogObjects_->frogEvent_->FindByDetId(EvtIdToFlip);
		if(!tmp)return;

		if(tmp->getDisplay()==0){
			tmp->setDisplay(1);
			EventToDisplay.push_back(EvtIdToFlip);
		}else{
			tmp->setDisplay(0);
			for(std::vector<unsigned int>::iterator it = EventToDisplay.begin();it!=EventToDisplay.end();it++){
				if(*it==EvtIdToFlip){
					EventToDisplay.erase(it);
					break;
				}
			}
		}
		frogObjects_->frogEvent_->setDisplay(2);
	}
}

void FROG_Events::keepOpenedMenu(FROG_Element_Base* mother=NULL){
	if(mother==NULL){
		mother = frogObjects_->frogEvent_;
		OpenedMenu.clear();		
	}

	FROG_Element_Base_With_DetId* FEBD = dynamic_cast<FROG_Element_Base_With_DetId*>(mother);
	if(mother->MenuOpened_==true && FEBD){		
		OpenedMenu.push_back(FEBD->DetId());
		for(unsigned int i=0;i<mother->getDaughterSize();i++){
			keepOpenedMenu(mother->getDaughter(i));
		}
	}else if(mother->type()<=C_EVENTRT && mother->type()>=C_EVENT){
		OpenedMenu.push_back(0);
		for(unsigned int i=0;i<mother->getDaughterSize();i++){
			keepOpenedMenu(mother->getDaughter(i));
		}
	}
}

void FROG_Events::updateOpenedMenu()
{	
	//printf("----------\n");
	for(unsigned int i=0;i<OpenedMenu.size();i++){		
		//printf("%i\n",OpenedMenu[i]);

		if(i==0 && OpenedMenu[i]==0){frogObjects_->frogEvent_->MenuOpened_ = true;}

		FROG_Element_Base* tmp = frogObjects_->frogEvent_->FindByDetId(OpenedMenu[i]);
		if(tmp){			
			tmp->MenuOpened_ = true;
		}
	}
}

