// Geometry.cpp: implementation of the Geometry class.
//
//////////////////////////////////////////////////////////////////////

#include "FROG_Geometry.h"
#include "FROG_Chunk.h"
#include "FROG_Element_Tools.h"
#include <algorithm>


void FROG_Geometry::Load (char* path)
{
	FILE* pFile = fopen(path,"rb" );
		if(pFile==NULL){
			printf("The file %s can not be opened !\n",path);
			return;
		}

		prim = new FROG_Element_Base(C_PRIMARY);
		FROG_ELEMENT::Read(pFile,prim,0);
		//FROG_ELEMENT::PrintTree(prim);
	fclose(pFile);

	FillMap(prim);
}


void FROG_Geometry::FillMap(FROG_Element_Base* elt) {
	const std::vector<FROG_Element_Base*>& dau = elt->daughters();	
	for( unsigned int i=0; i<dau.size(); ++i) {
		if( FROG_Element_Base_With_DetId *mod = dynamic_cast<FROG_Element_Base_With_DetId*>(dau[i])) {
			mapByDetId[mod->DetId()] = mod;
		}
		FillMap(dau[i]);
	}
}

void FROG_Geometry::Save (char* path)
{
	prim->write();
	FILE* pFile = fopen(path,"wb" );
		unsigned short prim_type = prim->type();
		unsigned int   prim_size = prim->size();
		fwrite(&prim_type,sizeof(prim_type),1,pFile);
		fwrite(&prim_size,sizeof(prim_size),1,pFile);
		fwrite(prim->data(),prim->size()-6,1,pFile);
    fclose(pFile);
	prim->clearData();
}

void FROG_Geometry::Save (char* path, bool CompressFiles){
	Save(path);

	#ifdef FROG_ZLIB
		if(CompressFiles){
			char* path2 = new char[strlen(path)];
			strcpy(path2,path);
			FROG_ZLIB_Deflate(path2);
			delete path2;
		}
	#endif
}


FROG_Element_Base* FROG_Geometry::FindByDetId (unsigned int DetId){
	Frog_map_it it = mapByDetId.find(DetId);
	if(it==mapByDetId.end())return NULL;
	return it->second;
}

void FROG_Geometry::updateGeomToDisplay(void* frogObjects_){
	FROG_ReadCards* Card = ((FROG_Objects*)frogObjects_)->frogCard_;
	if(Card->IsTagPresent("GeomToDisplay"))
		GeomToDisplay  = Card->GetUIntVector("GeomToDisplay");

	updateGeomToDisplay(GeomToDisplay);	 
	prim->init((void*)frogObjects_);
}

void FROG_Geometry::updateGeomToDisplay(vector<unsigned int> geometryToDisplay){
	prim->setDisplay(0);
	for(unsigned int i=0;i<geometryToDisplay.size();i++){
		FROG_Element_Base* tmp = FindByDetId(geometryToDisplay[i]);
		if(tmp)tmp->setDisplay(1);
	}
	prim->setDisplay(2);
}

void FROG_Geometry::display(){
	if(prim!=NULL){
		prim->display();
	}
}

