#ifndef __FROG_READCARDS_H
#define __FROG_READCARDS_H

#include <iostream>
#include <sstream>
#include <fstream>
#include <vector>
#include <string>
#include <stdio.h>
#include <stdarg.h>

using namespace std;

#ifdef linux
 #include <ext/hash_map>
 using namespace __gnu_cxx;

 namespace __gnu_cxx {
        template<> struct hash< std::string >{
                size_t operator()( const std::string& x ) const{ return hash< const char* >()( x.c_str() );  }
        };
 }

 typedef hash_map<std::string, std::vector< std::string >, hash<std::string>, equal_to<std::string> > Values;
 typedef hash_map<std::string, std::vector< std::string >, hash<std::string>, equal_to<std::string> >::iterator Values_Iterator;
#elif macos
 #include <ext/hash_map>
 using namespace __gnu_cxx;

 namespace __gnu_cxx {
        template<> struct hash< std::string >{
                size_t operator()( const std::string& x ) const{ return hash< const char* >()( x.c_str() );  }
        };
 }

 typedef hash_map<std::string, std::vector< std::string >, hash<std::string>, equal_to<std::string> > Values;
 typedef hash_map<std::string, std::vector< std::string >, hash<std::string>, equal_to<std::string> >::iterator Values_Iterator;
#else 
 #include <hash_map>
 using namespace stdext;
 typedef hash_map<std::string, std::vector< std::string >, hash_compare<std::string, std::less<std::string> > > Values;
 typedef hash_map<std::string, std::vector< std::string >, hash_compare<std::string, std::less<std::string> > >::iterator Values_Iterator;
#endif


/** Get tag-value pairs from a file.
 ** Reads in all the tag-value pairs from a file 
 ** (values can actually be a vector of values) and  
 ** makes them available by tagname.
 **
 **/

//typedef std::map < std::string, std::vector< std::string > >            Values;
//typedef std::map < std::string, std::vector< std::string > >::iterator  Values_Iteraor;

class FROG_ReadCards {

 public:

  FROG_ReadCards()                     {  }
  FROG_ReadCards(char* filename)       { Construct(filename); }
  FROG_ReadCards(std::string filename) { Construct(filename.c_str()); }

  ~FROG_ReadCards() { } 


  int GetNValues() const { return mValues.size(); }

  void Dump(FILE* file=stdout);

  bool IsTagPresent(const char* format, ...);
  std::vector<double> GetFloatVector(std::string tag);
  double GetFloat(const char* tag);
//  double GetFloat(const char* tag, double DefaultValue);
//  double GetFloat(double DefaultValue, const char* format, ...);
  void   GetFloat(float* Float, const char* format, ...);

  void GetColor(float* colorV, const char* format, ...);

  double GetInt(const char* tag);
  void GetInt(int* Int, const char* format, ...);
  void GetUInt(unsigned int* Int, const char* format, ...);
  void GetShort(short* Short, const char* format, ...);
  std::vector<int> GetIntVector(const char* format, ...);
  std::vector<unsigned int> GetUIntVector(const char* format, ...);
  void GetChar(char* Char, unsigned int sizeMax, const char* format, ...);

  double GetHex(const char* tag);
  std::string GetString(const char* format, ...);
  std::vector<string>  GetStringVector(const char* tag);
  bool GetBool(bool Default, const char* format, ...);


 private:
  
  void ReadParam();
  void clean();

  std::string  mFileName;  
  ifstream     mFile;
  int          mPos;
  std::string  mString;
  Values       mValues;

 private:

  void Construct(const char* filename);
  void print(); 
  std::string chop(std::string& s1, std::string s2);
  std::string chomp(std::string& s1, std::string s2);
  void removespace(std::string& s1);
  void parse();
  void error(const std::string s);

  std::vector<const char*> mFilesToRead;

   template< class T >
   double handle(const std::string& s, std::string format) const;
};

#endif










