// FROG_View.h: interface for the FROG_View class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _FROG_VIEW_H__
#define _FROG_VIEW_H__


#ifdef linux
	#include <GL/gl.h>
	#include <GL/glu.h> 
	#include "../GLUT/glut.h"
#elif macos
        #include <OpenGL/gl.h>
        #include <OpenGL/glu.h>
        #include <GLUT/glut.h>
#else		
	#include <windows.h> 
	#include <gl/gl.h>
	#include <gl/glu.h>
	#include <gl/glut.h>
#endif

#include "FROG_Objects_Extended.h"
#include "FROG_Camera.h"

class FROG_View{
public : 
	void (*displayFunc_)(void);
	bool		 IsActive;
	unsigned int Count_;
	unsigned int CountMax_;

	char  Type_;
	char* Name_;
	FROG_Objects_Extended*  frogObjects_;
	FROG_Camera*			cam_;

	unsigned int displayTexture_;
	unsigned int geom_display_list_;

	unsigned int ww_;
	unsigned int wh_;

	unsigned int v2w_;
	unsigned int v2h_;

	unsigned int vx_;
	unsigned int vy_;
	unsigned int vw_;
	unsigned int vh_;	

	float card_vx_;
	float card_vy_;
	float card_vw_;
	float card_vh_;	

	bool  screen_flag;

	float BackGround_Color_[4];
	vector<unsigned int>	geomToDisplay_;
public :	
	FROG_View(char* Name, FROG_Objects_Extended* frogObjects){
		displayFunc_    = NULL;
		displayTexture_ = 0;
		IsActive        = false;
		Count_			= 0;
		CountMax_       = 20+rand()%20;//10*rand()%20;


		Type_			= 0;
		Name_			= Name;
		frogObjects_	= frogObjects;
		cam_			= new FROG_Camera();
		
		cam_->R			= 700;
		cam_->phi		= 0;
		cam_->theta		= 0;

		card_vx_		= 0;
		card_vy_		= 0;
		card_vw_		= 1;
		card_vh_		= 1;

		screen_flag     = false;

		BackGround_Color_[0] = 0;
		BackGround_Color_[1] = 0;
		BackGround_Color_[2] = 0;
		BackGround_Color_[3] = 1;

		// Load all parameters from the card
		if(frogObjects_ && frogObjects_->frogCard_){
			frogObjects_->frogCard_->GetColor(BackGround_Color_ , "BackGround_Color");

			frogObjects_->frogCard_->GetFloat(&card_vx_			,"%s_Viewport_X"	,Name_);
			frogObjects_->frogCard_->GetFloat(&card_vy_			,"%s_Viewport_Y"	,Name_);
			frogObjects_->frogCard_->GetFloat(&card_vw_			,"%s_Viewport_W"	,Name_);
			frogObjects_->frogCard_->GetFloat(&card_vh_			,"%s_Viewport_H"	,Name_);

			frogObjects_->frogCard_->GetFloat(&(cam_->R)		,"%s_Cam_Pos_R"		,Name_);
			frogObjects_->frogCard_->GetFloat(&(cam_->theta)	,"%s_Cam_Pos_Theta"	,Name_);
			frogObjects_->frogCard_->GetFloat(&(cam_->phi)		,"%s_Cam_Pos_Phi"	,Name_);
			
			frogObjects_->frogCard_->GetFloat(&(cam_->target_x)	,"%s_Cam_Target_X"	,Name_);
			frogObjects_->frogCard_->GetFloat(&(cam_->target_y)	,"%s_Cam_Target_Y"	,Name_);
			frogObjects_->frogCard_->GetFloat(&(cam_->target_z)	,"%s_Cam_Target_Z"	,Name_);

			if(geomToDisplay_.size()>0)geomToDisplay_.clear();
			geomToDisplay_ = frogObjects_->frogCard_->GetUIntVector("%s_Geom"		,Name_);
			
			if(frogObjects_ && frogObjects_->frogGeom_ && geomToDisplay_.size()>0){			
				frogObjects_->frogGeom_->updateGeomToDisplay(geomToDisplay_);
				frogObjects_->frogGeom_->prim->init((void*)frogObjects_);

				if(geom_display_list_>0){
					glDeleteLists(geom_display_list_, 1);
					geom_display_list_=0;
				}

				geom_display_list_ = glGenLists (1);
				glNewList(geom_display_list_, GL_COMPILE_AND_EXECUTE);
					glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
					glDisable(GL_CULL_FACE);
					for(unsigned int i=0;i<geomToDisplay_.size();i++){
						FROG_Element_Base* tmp = frogObjects_->frogGeom_->FindByDetId(geomToDisplay_[i]);
						if(!tmp)continue;			
						tmp->display(false);
					}
					glEnable(GL_CULL_FACE);
				glEndList();	
			}			
		}
	}

	virtual ~FROG_View(){}

	virtual bool IsSwapPossible(){return true;}
	virtual bool Is2D(){return false;}
	virtual bool Is3D(){return false;}

	virtual void Reshape(unsigned int w, unsigned int h){
		if(w!=0 || h!=0){
			ww_ = w;
			wh_ = h;			
		}

		if(screen_flag){			
			vx_ = 0;
			vy_ = 0;
			vw_ = ww_;
			vh_ = wh_;
		}else{
			vx_ = (unsigned int) (card_vx_ * ww_);
			vy_ = (unsigned int) (card_vy_ * wh_);
			vw_ = (unsigned int) (card_vw_ * ww_);
			vh_ = (unsigned int) (card_vh_ * wh_);
		}
		MarkForUpdate();
	}

	virtual void SetActive(){IsActive = true;}
	virtual void Display_Begin(){};
	virtual void Display_End(){};
	virtual void Display_Core(){};

	virtual bool IsInViewPort(unsigned int x, unsigned int y){
		y = wh_-y; // Is there because glviewport works with (0,0) in bottom left instead of top left like in standard function.

		if(x>vx_ && y>vy_&& x<vx_+vw_ && y<vy_ + vh_) return true;
		return false;
	}

	virtual void ProjectionMatrix(){}

	virtual void DisplayBkgGeom(){
		if(geom_display_list_>0){
			glCallList(geom_display_list_);
		}
	}

	virtual void Display(){}

	virtual void MarkForUpdate(){
		if(displayTexture_>0){
		  glDeleteTextures(1,(GLuint*)&displayTexture_);
			displayTexture_ = 0;
		}		
	}

	void FullScreen(){
		screen_flag = !screen_flag;
		Reshape(ww_,wh_);	
	}

	virtual void ClearBackground(){		
		glViewport (vx_, vy_, vw_, vh_);
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		glOrtho(0, vw_, 0, vh_, -1.0f, 1.0f);
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();
		glPushAttrib(GL_DEPTH_TEST);
		glDisable(GL_DEPTH_TEST);
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		glColor4fv (BackGround_Color_);
		glBegin (GL_QUADS);
			glVertex2i (0   , 0  );
			glVertex2i (vw_	, 0  );
			glVertex2i (vw_	, vh_);
			glVertex2i (0   , vh_);
		glEnd (); 		

		glPopAttrib();
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();	
	}


	void DisplayTexture(){
			glViewport (vx_, vy_, (GLsizei)vw_, (GLsizei)vh_);

			glMatrixMode(GL_PROJECTION);
			glPushMatrix();
			glLoadIdentity();
			glOrtho(0, vw_, 0, vh_, -1.0f, 1.0f);
			//glOrtho(vx_, vw_, vy_, vh_, -1.0f, 1.0f);
			glMatrixMode(GL_MODELVIEW);
			glPushMatrix();
			glLoadIdentity();
			glPushAttrib(GL_DEPTH_TEST);
			glDisable(GL_DEPTH_TEST);			

			glBindTexture(GL_TEXTURE_2D,displayTexture_);
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			glEnable(GL_TEXTURE_2D);

			glColor4f (1,1,1,1);
			glBegin (GL_QUADS);
				glTexCoord2i(0,0); glVertex3i (0  , 0  , -1);
				glTexCoord2i(1,0); glVertex3i (v2w_, 0  , -1);
				glTexCoord2i(1,1); glVertex3i (v2w_, v2h_, -1);
				glTexCoord2i(0,1); glVertex3i (0  , v2h_, -1);
			glEnd (); 
			
			glDisable(GL_TEXTURE_2D);
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
			glBindTexture(GL_TEXTURE_2D,0);

			glPopAttrib();
			glMatrixMode(GL_PROJECTION);
			glPopMatrix();
			glMatrixMode(GL_MODELVIEW);
			glPopMatrix();

			Count_++;
			if(Count_==CountMax_){
				MarkForUpdate();				
			}	
	}

	void RecordTexture(){
		v2w_ = 1; while(v2w_<vw_)v2w_ = v2w_<<1;
		v2h_ = 1; while(v2h_<vh_)v2h_ = v2h_<<1;
		Count_=0;

		if(displayTexture_>0)glDeleteTextures(1,(GLuint*)&displayTexture_);
		glGenTextures  (1, (GLuint*)&displayTexture_);
		glBindTexture  (GL_TEXTURE_2D, displayTexture_);
		glTexImage2D   (GL_TEXTURE_2D, 0, GL_RGBA, v2w_,v2h_, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 			
		glCopyTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,vx_, vy_,v2w_,v2h_,0);			
	}
};


#endif


