// FROG_View.h: interface for the FROG_View class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _FROG_VIEW_2D_H__
#define _FROG_VIEW_2D_H__

#include "FROG_View.h"
#include "FROG_Texture.h"

class FROG_View_2D : public FROG_View{
public :	
	float  sliceDepth; 
	double orthoXMin_;
	double orthoXMax_;
	double orthoYMin_;
	double orthoYMax_;
	double orthoZMin_;
	double orthoZMax_;
public :
	FROG_View_2D(char* Name, FROG_Objects_Extended* frogObjects) : FROG_View(Name, frogObjects){
		Type_      = 2;
		sliceDepth = 1000;

		if(frogObjects_ && frogObjects_->frogCard_){
			frogObjects_->frogCard_->GetFloat(&sliceDepth		,"%s_Slice_Depth"	,Name_);
		}
	}
	virtual bool Is2D(){return true;}
	virtual bool Is3D(){return false;}

	virtual void ProjectionMatrix(){
		glOrtho(orthoXMin_,orthoXMax_,orthoYMin_,orthoYMax_,orthoZMin_,orthoZMax_);		
	}

	virtual void Display_Begin(){
		orthoXMin_ = -1*cam_->R;
		orthoXMax_ =  1*cam_->R;
		orthoYMin_ = orthoXMin_ * ((double)vh_/vw_);
		orthoYMax_ = orthoXMax_ * ((double)vh_/vw_);
		orthoZMin_ = cam_->R - sliceDepth;
		orthoZMax_ = cam_->R + sliceDepth;

		glViewport (vx_, vy_, (GLsizei)vw_, (GLsizei)vh_);

		glMatrixMode (GL_PROJECTION);
		glLoadIdentity ();
		ProjectionMatrix();

		glMatrixMode (GL_MODELVIEW);
		glLoadIdentity ();	

		cam_->LookAt();
	}


	virtual void Display(){		
		if(!displayFunc_)return;

		if(displayTexture_>0 && !IsActive){
			DisplayTexture();
		}else{						
			ClearBackground();
			Display_Begin();

			DisplayBkgGeom();		
			Display_Core();
			if(!IsActive)RecordTexture();

			Display_End();			
		}
	}

	virtual void Display_Core(){
		if(frogObjects_ && frogObjects_->selectedObject_ && frogObjects_->selectedObject_DisplayFlag){
			if(frogObjects_->IsSelected_Geometry()){glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);}else{glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);}
			float color[4] = {1 - frogObjects_->selectedObject_->style_->color_[0],1 - frogObjects_->selectedObject_->style_->color_[1],1 - frogObjects_->selectedObject_->style_->color_[2],1};						
			frogObjects_->selectedObject_->display(true, color);
		}

		displayFunc_();
	}
};


#endif

