// FROG_View.h: interface for the FROG_View class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _FROG_VIEW_SCREEN_H__
#define _FROG_VIEW_SCREEN_H__

#include "FROG_View.h"

class FROG_View_Screen : public FROG_View{
public : 
	unsigned int FPS;
	unsigned int FPS_Start;
	unsigned int FPS_Current;
	unsigned int FPS_N;

public :
	FROG_View_Screen(char* Name, FROG_Objects_Extended* frogObjects=NULL) : FROG_View(Name, frogObjects){
		Type_     = 1;
		FPS       = 0;
		FPS_Start = clock();		
	}

	virtual void UpdateFPS(){
		FPS_N++;
		FPS_Current = clock();
		if(FPS_Current>FPS_Start+CLOCKS_PER_SEC){
			//printf("FPS = %f\n",(1.0*(FPS_N*CLOCKS_PER_SEC)) / (FPS_Current-FPS_Start));
			FPS       = (FPS_N*CLOCKS_PER_SEC) / (FPS_Current-FPS_Start);			
			FPS_N     = 0;	
			FPS_Start = FPS_Current;
		}
	}

	void Display_Begin(){		
		glViewport (0, 0, ww_, wh_);
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		glOrtho(0, ww_, 0, wh_, -1.0f, 1.0f);
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();
		glPushAttrib(GL_DEPTH_TEST);
		glDisable(GL_DEPTH_TEST);
	}

	void Display_End(){
		glPopAttrib();
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();	
	}

	virtual void Display(){
		if(!displayFunc_)return;
		
		UpdateFPS();
		Display_Begin();
		displayFunc_();
		Display_End();	
	}

	void DrawLogo(int X, int Y, int W, int H, GLuint TextID, FROG_Fonts* Fonts)
	{
		if( ((float)(W)) / H != 1.5){
			if(((float)(W))>1.5*H){
				W = (int)(1.5*H);
			}else{
				H = (int)(W/1.5);
			}			
		}

		Display_Begin();

		glBindTexture(GL_TEXTURE_2D,TextID);
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		glEnable(GL_TEXTURE_2D);

		glColor4f (1,1,1,1);
		glBegin (GL_QUADS);
			glTexCoord2i(0,0); glVertex3i (X  , Y  , -1);
			glTexCoord2i(1,0); glVertex3i (X+W, Y  , -1);
			glTexCoord2i(1,1); glVertex3i (X+W, Y+H, -1);
			glTexCoord2i(0,1); glVertex3i (X  , Y+H, -1);
		glEnd (); 
		
		glDisable(GL_TEXTURE_2D);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		glBindTexture(GL_TEXTURE_2D,0);

		Fonts->Font_Times_Roman_10->glPrint(0, 3,"Version = %6.3f",FROG_VERSION);

		Display_End();
	}

	void LoadingInProgress(float Pourcentage, char* txt, FROG_Fonts* Fonts, GLuint TextID)
	{
		if(txt!=NULL)
		printf("Loading %04.1f%% : %s\n",Pourcentage,txt);

		Display_Begin();


		glColor4f (1,1,1,1);
		glBegin (GL_QUADS);
			glVertex2i (0   , 0  );
			glVertex2i (ww_	, 0  );
			glVertex2i (ww_	, wh_);
			glVertex2i (0   , wh_);
		glEnd (); 

		glBindTexture(GL_TEXTURE_2D,TextID);	
		glEnable(GL_TEXTURE_2D);

		int X = (int)(0.2*ww_);
		int Y = (int)(0.35*wh_);
		int W = (int)(0.6*ww_);
		int H = (int)(0.6*wh_);

		glColor4f (1,1,1,1);
		glBegin (GL_QUADS);
			glTexCoord2i(0,0); glVertex3i ( X   ,  Y   , -1);
			glTexCoord2i(1,0); glVertex3i ((X+W),  Y   , -1);
			glTexCoord2i(1,1); glVertex3i ((X+W), (Y+H), -1);
			glTexCoord2i(0,1); glVertex3i ( X   , (Y+H), -1);
		glEnd (); 

		glDisable(GL_TEXTURE_2D);	
		glBindTexture(GL_TEXTURE_2D,0);
		
		glColor4f (0,0,0,1);
		Fonts->Font_Times_Roman_24->glPrint(X, Y-60 ,"Fast and Realistic OpenGl Event Display : FROG");
		
		Fonts->Font_Fixed_9_by_15->glPrint(X, Y-140, "Authors List : ");
		Fonts->Font_Fixed_9_by_15->glTextPrint(X+140, Y-140, FROG_AUTHORLIST);	
		Fonts->Font_Times_Roman_10->glPrint(3, 3 ,"Version = %6.3f of %s",FROG_VERSION,__DATE__);

		glColor4f (0,0.6,0,1);
		if(Pourcentage>=0){		
			Fonts->Font_Helvetica_18->glPrint((int)(0.38*ww_), (int)(0.80*wh_), "Loading %04.1f%%", Pourcentage);					

			if(txt!=NULL)
				Fonts->Font_Helvetica_18->glTextPrint((int)(0.31*ww_), (int)(0.74*wh_), txt);		
		}

		Display_End();
		fflush(stdout);
		glutSwapBuffers();
	}

};


#endif

